@@ -4,17 +4,17 @@ import sys
 import unittest
 import os
 import tempfile
-import numpy as NP
+import warnings
+
+import numpy as np
+from collections import OrderedDict
 from numpy.random.mtrand import uniform
+
 import netCDF4
 
-try:
-    from collections import OrderedDict
-except ImportError: # or else use drop-in substitute
-    from ordereddict import OrderedDict
-
 # test attribute creation.
-FILE_NAME = tempfile.mktemp(".nc")
+#FILE_NAME = tempfile.NamedTemporaryFile(suffix='.nc', delete=False).name
+FILE_NAME = 'tst_atts.nc'
 VAR_NAME="dummy_var"
 GROUP_NAME = "dummy_group"
 DIM1_NAME="x"
@@ -27,10 +27,13 @@ STRATT = 'string attribute'
 EMPTYSTRATT = ''
 INTATT = 1
 FLOATATT = math.pi
-SEQATT = NP.arange(10)
-STRINGSEQATT = ['mary ','had ','a ','little ','lamb']
+SEQATT = np.arange(10)
+STRINGSEQATT = ['mary ','','had ','a ','little ','lamb',]
+#ATTDICT = {'stratt':STRATT,'floatatt':FLOATATT,'seqatt':SEQATT,
+#           'stringseqatt':''.join(STRINGSEQATT), # changed in issue #770
+#           'emptystratt':EMPTYSTRATT,'intatt':INTATT}
 ATTDICT = {'stratt':STRATT,'floatatt':FLOATATT,'seqatt':SEQATT,
-           'stringseqatt':''.join(STRINGSEQATT),
+           'stringseqatt':STRINGSEQATT,
            'emptystratt':EMPTYSTRATT,'intatt':INTATT}
 
 class VariablesTestCase(unittest.TestCase):
@@ -49,6 +52,7 @@ class VariablesTestCase(unittest.TestCase):
         f.seqatt = SEQATT
         # sequences of strings converted to a single string.
         f.stringseqatt = STRINGSEQATT
+        f.setncattr_string('stringseqatt_array',STRINGSEQATT) # array of NC_STRING
         g = f.createGroup(GROUP_NAME)
         f.createDimension(DIM1_NAME, DIM1_LEN)
         f.createDimension(DIM2_NAME, DIM2_LEN)
@@ -63,6 +67,10 @@ class VariablesTestCase(unittest.TestCase):
         g.floatatt = FLOATATT
         g.seqatt = SEQATT
         g.stringseqatt = STRINGSEQATT
+        if netCDF4.__version__ > "1.4.2":
+            with self.assertRaises(ValueError):
+                g.arrayatt = [[1, 2], [3, 4]] # issue #841
+        g.setncattr_string('stringseqatt_array',STRINGSEQATT) # array of NC_STRING
         v = f.createVariable(VAR_NAME, 'f8',(DIM1_NAME,DIM2_NAME,DIM3_NAME))
         # try to set a variable attribute with one of the reserved names.
         v.setncattr('ndim','three')
@@ -75,6 +83,7 @@ class VariablesTestCase(unittest.TestCase):
         v.floatatt = FLOATATT
         v.seqatt = SEQATT
         v.stringseqatt = STRINGSEQATT
+        v.setncattr_string('stringseqatt_array',STRINGSEQATT) # array of NC_STRING
         v1 = g.createVariable(VAR_NAME, 'f8',(DIM1_NAME,DIM2_NAME,DIM3_NAME))
         v1.stratt = STRATT
         v1.emptystratt = EMPTYSTRATT
@@ -82,22 +91,49 @@ class VariablesTestCase(unittest.TestCase):
         v1.floatatt = FLOATATT
         v1.seqatt = SEQATT
         v1.stringseqatt = STRINGSEQATT
+        v1.setncattr_string('stringseqatt_array',STRINGSEQATT) # array of NC_STRING
+        # issue #959: should not be able to set _FillValue after var creation
+        try:
+            v1._FillValue(-999.)
+        except AttributeError:
+            pass
+        else:
+            raise ValueError('This test should have failed.')
+        try:
+            v1.setncattr('_FillValue',-999.)
+        except AttributeError:
+            pass
+        else:
+            raise ValueError('This test should have failed.')
         # issue #485 (triggers segfault in C lib
         # with version 1.2.1 without pull request #486)
-        f.foo = NP.array('bar','S')
-        f.foo = NP.array('bar','U')
+        f.foo = np.array('bar','S')
+        f.foo = np.array('bar','U')
         # issue #529 write string attribute as NC_CHAR unless
         # it can't be decoded to ascii.  Add setncattr_string
         # method to force NC_STRING.
-        f.charatt = u'foo' # will be written as NC_CHAR
+        f.charatt = 'foo' # will be written as NC_CHAR
         f.setncattr_string('stringatt','bar') # NC_STRING
-        f.cafe = u'caf\xe9' # NC_STRING
-        f.batt = u'caf\xe9'.encode('utf-8') #NC_CHAR
+        f.cafe = 'caf\xe9' # NC_STRING
+        f.batt = 'caf\xe9'.encode('utf-8') #NC_CHAR
         v.setncattr_string('stringatt','bar') # NC_STRING
+        # issue #882 - provide an option to always string attribute
+        # as NC_STRINGs. Testing various approaches to setting text attributes...
+        f.set_ncstring_attrs(True)
+        f.stringatt_ncstr = 'foo' # will now be written as NC_STRING
+        f.setncattr_string('stringatt_ncstr','bar') # NC_STRING anyway
+        f.caf_ncstr = 'caf\xe9' # NC_STRING anyway
+        f.bat_ncstr = 'caf\xe9'.encode('utf-8') # now NC_STRING
+        g.stratt_ncstr = STRATT # now NC_STRING
+        #g.renameAttribute('stratt_tmp','stratt_ncstr')
+        v.setncattr_string('stringatt_ncstr','bar') # NC_STRING anyway
+        v.stratt_ncstr = STRATT
+        v1.emptystratt_ncstr = EMPTYSTRATT
         f.close()
 
     def tearDown(self):
         # Remove the temporary files
+        #pass
         os.remove(self.file)
 
     def runTest(self):
@@ -110,7 +146,7 @@ class VariablesTestCase(unittest.TestCase):
         # global attributes.
         # check __dict__ method for accessing all netCDF attributes.
         for key,val in ATTDICT.items():
-            if type(val) == NP.ndarray:
+            if type(val) == np.ndarray:
                 assert f.__dict__[key].tolist() == val.tolist()
             else:
                 assert f.__dict__[key] == val
@@ -120,12 +156,14 @@ class VariablesTestCase(unittest.TestCase):
         assert f.stratt == STRATT
         assert f.emptystratt == EMPTYSTRATT
         assert f.seqatt.tolist() == SEQATT.tolist()
-        assert f.stringseqatt == ''.join(STRINGSEQATT)
+        #assert f.stringseqatt == ''.join(STRINGSEQATT) # issue 770
+        assert f.stringseqatt == STRINGSEQATT
+        assert f.stringseqatt_array == STRINGSEQATT
         assert f.getncattr('file_format') == 'netcdf4_format'
         # variable attributes.
         # check __dict__ method for accessing all netCDF attributes.
         for key,val in ATTDICT.items():
-            if type(val) == NP.ndarray:
+            if type(val) == np.ndarray:
                 assert v.__dict__[key].tolist() == val.tolist()
             else:
                 assert v.__dict__[key] == val
@@ -134,26 +172,27 @@ class VariablesTestCase(unittest.TestCase):
         assert v.floatatt == FLOATATT
         assert v.stratt == STRATT
         assert v.seqatt.tolist() == SEQATT.tolist()
-        assert v.stringseqatt == ''.join(STRINGSEQATT)
+        #assert v.stringseqatt == ''.join(STRINGSEQATT) # issue 770
+        assert v.stringseqatt == STRINGSEQATT
+        assert v.stringseqatt_array == STRINGSEQATT
         assert v.getncattr('ndim') == 'three'
         assert v.getncattr('foo') == 1
         assert v.getncattr('bar') == 2
         # check type of attributes using ncdump (issue #529)
-        dep=subprocess.Popen(['ncdump','-h',FILE_NAME],stdout=subprocess.PIPE).communicate()[0]
-        try: # python 2
-            ncdump_output = dep.split('\n')
-        except TypeError: # python 3
-            ncdump_output = str(dep,encoding='utf-8').split('\n')
-        for line in ncdump_output:
-            line = line.strip('\t\n\r')
-            if "stringatt" in line: assert line.startswith('string')
-            if "charatt" in line: assert line.startswith(':')
-            if "cafe" in line: assert line.startswith('string')
-            if "batt" in line: assert line.startswith(':')
+        if not os.getenv('NO_CDL'):
+            ncdump_output = f.tocdl()
+            for line in ncdump_output:
+                line = line.strip('\t\n\r')
+                line = line.strip()# Must be done another time for group variables
+                if "stringatt" in line: assert line.startswith('string')
+                if "charatt" in line: assert line.startswith(':')
+                if "cafe" in line: assert line.startswith('string')
+                if "batt" in line: assert line.startswith(':')
+                if "_ncstr" in line: assert line.startswith('string')
         # check attributes in subgroup.
         # global attributes.
         for key,val in ATTDICT.items():
-            if type(val) == NP.ndarray:
+            if type(val) == np.ndarray:
                 assert g.__dict__[key].tolist() == val.tolist()
             else:
                 assert g.__dict__[key] == val
@@ -162,9 +201,11 @@ class VariablesTestCase(unittest.TestCase):
         assert g.stratt == STRATT
         assert g.emptystratt == EMPTYSTRATT
         assert g.seqatt.tolist() == SEQATT.tolist()
-        assert g.stringseqatt == ''.join(STRINGSEQATT)
+        #assert g.stringseqatt == ''.join(STRINGSEQATT) # issue 770
+        assert g.stringseqatt == STRINGSEQATT
+        assert g.stringseqatt_array == STRINGSEQATT
         for key,val in ATTDICT.items():
-            if type(val) == NP.ndarray:
+            if type(val) == np.ndarray:
                 assert v1.__dict__[key].tolist() == val.tolist()
             else:
                 assert v1.__dict__[key] == val
@@ -173,9 +214,15 @@ class VariablesTestCase(unittest.TestCase):
         assert v1.stratt == STRATT
         assert v1.emptystratt == EMPTYSTRATT
         assert v1.seqatt.tolist() == SEQATT.tolist()
-        assert v1.stringseqatt == ''.join(STRINGSEQATT)
+        #assert v1.stringseqatt == ''.join(STRINGSEQATT) # issue 770
+        assert v1.stringseqatt == STRINGSEQATT
+        assert v1.stringseqatt_array == STRINGSEQATT
         assert getattr(v1,'nonexistantatt',None) == None
         f.close()
+        # issue 915 empty string attribute (ncdump reports 'NIL')
+        f = netCDF4.Dataset('test_gold.nc')
+        assert f['RADIANCE'].VAR_NOTES == ""
+        f.close()
 
 if __name__ == '__main__':
     unittest.main()
